
require "mkmf"

dir_config('x')

$objs  = ["tgif.o", "tgif_lib.o"]
$objs += Dir.glob("../*.o")
$objs -= ["../tgif.o"]

#have_library("socket", "connect")
have_library("X11", "XOpenDisplay")

create_makefile("tgif")
