
#ifndef _TGIF_LIB_H_
#define _TGIF_LIB_H_

extern int	InitTgifInterface ARGS_DECL((void));
extern int	TgifLoadFile ARGS_DECL((char *FullName));
extern int	TgifOutObj ARGS_DECL((FILE*, char *psz_format));
extern int	TgifOutPS ARGS_DECL((char *filename, char *psz_format));

extern int	TgifBox ARGS_DECL((
			float *ltx, float *lty, float *rbx, float *rby,
			int rotate));   /* 0: no rotate, +1: 45, -1: 135 */
extern int	TgifRCBox ARGS_DECL((
			float *ltx, float *lty, float *rbx, float *rby,
			int rotate));   /* 0: no rotate, +1: 45, -1: 135 */
extern int	TgifOval ARGS_DECL((
			float *ltx, float *lty, float *rbx, float *rby));
extern int	TgifStartVertex ARGS_DECL((float *x, float *y));
extern int	TgifAddVertex ARGS_DECL((float *x, float *y));
extern int	TgifPoly ARGS_DECL((void));
extern int	TgifPolygon ARGS_DECL((void));
extern int	TgifArc ARGS_DECL((
			float *xc, float *yc, float *x1, float *y1, float *x2, float *y2, float *dir,
			float *ltx, float *lty, float *w, float *h, float *angle1, float *angle2));
extern int	TgifText ARGS_DECL((
			float *x,
			float *baseline_y,
			char *font_name,
			int font_size,
			char *buf));

extern int	TgifImportGIF ARGS_DECL((
			char *path, float *x, float *y, float *w, float *h,
			char *gamma, int fit));

extern int	TgifImportJPEG ARGS_DECL((
			char *path, float *x, float *y, float *w, float *h,
			char *gamma,
			int fit)); /* 0: don't fit, 1: stretch to fit */

extern void TgifSaveState ARGS_DECL((void));
extern void TgifRestoreState ARGS_DECL((void));

#endif /* _TGIF_LIB_H_ */
