#!/usr/bin/env ruby

require "tgif"

Tgif.new

Tgif.text(72 * 3, 72 * 2, "こんにちは", "Ryumin-Light-EUC-H", 34)
Tgif.text(72 * 3, 72 * 3, "こんにちは", "GothicBBB-Medium-EUC-H", 34)

f = open("test01_ja.obj", "w")
Tgif.out(f, "obj")
f.close

Tgif.dump("test01_ja.ps", "ps")

Tgif.delete
